import time
from dataEngine import AppConfig, Logger
from dataEngine.Driver import BaseConnector
from dataEngine.cw import EventHandlers, Ldap
from asterisk.ami import AMIClient, AutoReconnect, SimpleAction

logger = Logger.get()

class Connector(BaseConnector):
    AMI_CLIENT = None

    def __init__(self):
        super(BaseConnector, self).__init__()
        pass

    def init_connection(self, cleanup_all_data):
        port = AppConfig.get("port")
        if self.AMI_CLIENT is None:
            self.AMI_CLIENT = AMIClient(address='127.0.0.1', port=port)

        def on_disconnect(client, error):
            if error is not None:
                logger.error("Connection error: {}".format(error))
                self.AMI_CLIENT._futures = {}
                cleanup_all_data()

        AutoReconnect(ami_client=self.AMI_CLIENT, delay=5, on_disconnect=on_disconnect)

        try:
            if AppConfig.isTest():
                time.sleep(0.3)
            self.AMI_CLIENT.login(username='root', secret='wildix')
        except Exception as e:
            logger.error("Can't connect to AMI port {}".format(port))
            self.AMI_CLIENT = None
            AppConfig.set("exit_code", 1)
            AppConfig.set("stop", True)
            return
        # self.AMI_CLIENT.add_event_listener(event_listener, white_list=[
        #     "CEL",
        #     "MonitorStart",
        #     "MonitorStop",
        #     "UniqueIDMasquerade",
        #       "MusicOnHold",
        #       "BRIDGE_START",
        #       "BRIDGE_STOP",
        # ])
        self.AMI_CLIENT.add_event_listener(EventHandlers.event_listener, black_list=[
            "DeviceState",
            "Newchannel",
            "ChannelUpdate",
            "Newstate",
            "EventConnectedLine",
            "Bridge",
            "NewCallerid",
            "ExtensionStatus",
            "NewAccountCode",
            "HangupRequest",
            "SoftHangupRequest",
            "PeerStatus",
            "Cdr",
            "Hold",
            "QueueMemberStatus",
            "Dial",
            "Masquerade",
            "Rename",
            "BridgeAction",
            "Hangup",
            "EventFeature"  # {'Privilege': 'call,all', 'Device': '102', 'Feature': 'AWAY', 'State': 'false'}
        ])


    def send_action(self, action_name, **kwargs):
        if not action_name:
            return "No action name"
        elif AppConfig.checkMode("none"):
            return "Connect to the AMI disabled by configuration"
        elif not self.AMI_CLIENT:
            return "AMI connector didn't initiate"
        try:
            future = self.AMI_CLIENT.send_action(SimpleAction(action_name, **kwargs))
            return str(future.response)
        except Exception as e:
            logger.error(e)
            return str(e)


    def health_check(self):
        res = self.send_action("Ping")
        if res and res.find("Pong") >= 0:
            return True
        return False


    def close_connection(self):
        if self.AMI_CLIENT:
            self.AMI_CLIENT.logoff()
        Ldap.store()


    def get_my_port(self):
        return self.AMI_CLIENT._socket.getsockname()[1]
